@@ -26,15 +26,24 @@
 #include <glib/gstdio.h>
 #include <pwd.h>
 #include <gio/gio.h>
-#include <libgnomeui/gnome-desktop-thumbnail.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <pwd.h>
 #include <unistd.h>
 #include <libebook/e-book.h>
+#include <dbus/dbus-glib-bindings.h>
+
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnomeui/gnome-desktop-thumbnail.h>
+
+#ifdef HAVE_POLKIT
+#include <polkit-gnome/polkit-gnome.h>
+#endif
 
 #include "e-image-chooser.h"
 #include "gnome-about-me-password.h"
+#include "gnome-about-me-fingerprint.h"
+#include "marshal.h"
 
 #include "capplet-util.h"
 
@@ -48,6 +57,8 @@ typedef struct {
 	EBook    	*book;
 
 	GladeXML 	*dialog;
+	GtkWidget	*enable_fingerprint_button;
+	GtkWidget	*disable_fingerprint_button;
 
 	GdkScreen    	*screen;
 	GtkIconTheme 	*theme;
@@ -156,6 +167,27 @@ about_me_error (GtkWindow *parent, gchar *str)
 	gtk_widget_destroy (dialog);
 }
 
+#ifdef HAVE_POLKIT
+static GtkWidget *
+create_fingerprint_button (const char *msg, const char *name)
+{
+	GtkWidget *button;
+	PolKitAction *action;
+	PolKitGnomeAction *gaction;
+
+	action = polkit_action_new_from_string_representation ("net.reactivated.fprint.device.enroll");
+	gaction = polkit_gnome_action_new_default (name,
+						   action,
+						   msg,
+						   NULL);
+	polkit_action_unref (action);
+
+	button = polkit_gnome_action_create_button (gaction);
+	g_object_unref (gaction);
+
+	return button;
+}
+#endif /* HAVE_POLKIT */
 
 /********************/
 static void
@@ -303,6 +335,7 @@ about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 			me->email[ids[i].cid] = NULL;
 		else
 			me->email[ids[i].cid] = g_strdup (str);
+		me->email_types[ids[i].cid] = (i == EMAIL_HOME) ? ATTRIBUTE_HOME : ATTRIBUTE_WORK;
 	/* FIXME: i'm getting an empty address field in evolution */
 	} else if (i >= ADDRESS_HOME && i < ADDRESS_WORK) {
 		about_me_set_address_field (me->addr1, ids[i].cid, str);
@@ -523,7 +556,8 @@ about_me_load_photo (GnomeAboutMe *me, EContact *contact)
 
 	if (photo && photo->type == E_CONTACT_PHOTO_TYPE_INLINED) {
 		me->have_image = TRUE;
-		e_image_chooser_set_image_data (E_IMAGE_CHOOSER (widget), photo->data.inlined.data, photo->data.inlined.length);
+		e_image_chooser_set_image_data (E_IMAGE_CHOOSER (widget),
+						(char *) photo->data.inlined.data, photo->data.inlined.length);
 		e_contact_photo_free (photo);
 	} else {
 		me->have_image = FALSE;
@@ -588,7 +622,7 @@ about_me_update_photo (GnomeAboutMe *me)
 						   "compression", "9", NULL);
 
 			g_free (data);
-			data = scaled_data;
+			data = (guchar *) scaled_data;
 			length = scaled_length;
 		}
 
@@ -603,7 +637,7 @@ about_me_update_photo (GnomeAboutMe *me)
 		error = NULL;
 		file = g_build_filename (g_get_home_dir (), ".face", NULL);
 		if (g_file_set_contents (file,
-					 photo->data.inlined.data,
+					 (gchar *) photo->data.inlined.data,
 					 photo->data.inlined.length,
 					 &error) != FALSE) {
 			g_chmod (file, 0644);
@@ -650,6 +684,9 @@ about_me_load_info (GnomeAboutMe *me)
 	for (i = 0; ids[i].wid != NULL; i++) {
 		about_me_load_string_field (me, ids[i].wid, ids[i].cid, i);
 	}
+
+	set_fingerprint_label (me->enable_fingerprint_button,
+			       me->disable_fingerprint_button);
 }
 
 GtkWidget *
@@ -856,6 +893,14 @@ about_me_passwd_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
 	gnome_about_me_password (GTK_WINDOW (WID ("about-me-dialog")));
 }
 
+static void
+about_me_fingerprint_button_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
+{
+	fingerprint_button_clicked (me->dialog,
+				    me->enable_fingerprint_button,
+				    me->disable_fingerprint_button);
+}
+
 static gint
 about_me_setup_dialog (void)
 {
@@ -923,7 +968,7 @@ about_me_setup_dialog (void)
 		if (me->book == NULL) {
 			me->book = e_book_new_system_addressbook (&error);
 			if (me->book == NULL || error != NULL) {
-				g_error ("%s\n", error->message);
+				g_error ("%s", error->message);
 				g_clear_error (&error);
 			}
 
@@ -989,6 +1034,25 @@ about_me_setup_dialog (void)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (about_me_image_clicked_cb), me);
 
+#ifdef HAVE_POLKIT
+	gtk_widget_destroy (WID ("enable_fingerprint_button"));
+	gtk_widget_destroy (WID ("disable_fingerprint_button"));
+
+	me->enable_fingerprint_button = create_fingerprint_button (_("Enable _Fingerprint Login..."), "enable-action");
+	gtk_container_add (GTK_CONTAINER (WID("buttons_vbox")), me->enable_fingerprint_button);
+
+	me->disable_fingerprint_button = create_fingerprint_button (_("Disable _Fingerprint Login..."), "disable-action");
+	gtk_container_add (GTK_CONTAINER (WID("buttons_vbox")), me->disable_fingerprint_button);
+#else
+	me->enable_fingerprint_button = WID ("enable_fingerprint_button");
+	me->disable_fingerprint_button = WID ("disable_fingerprint_button");
+#endif /* HAVE_POLKIT */
+
+	g_signal_connect (me->enable_fingerprint_button, "clicked",
+			  G_CALLBACK (about_me_fingerprint_button_clicked_cb), me);
+	g_signal_connect (me->disable_fingerprint_button, "clicked",
+			  G_CALLBACK (about_me_fingerprint_button_clicked_cb), me);
+
 	widget = WID ("image-chooser");
 	g_signal_connect (widget, "changed",
 			  G_CALLBACK (about_me_image_changed_cb), me);
@@ -1027,6 +1091,8 @@ main (int argc, char **argv)
 	int rc = 0;
 
 	capplet_init (NULL, &argc, &argv);
+	dbus_g_object_register_marshaller (fprintd_marshal_VOID__STRING_BOOLEAN,
+					   G_TYPE_NONE, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INVALID);
 
 	rc = about_me_setup_dialog ();
 
