@@ -29,6 +29,7 @@
 #include <gdk/gdkx.h>
 #include <X11/Xft/Xft.h>
 #endif /* HAVE_XFT2 */
+#include <glib/gi18n.h>
 
 #include "capplet-util.h"
 #include "gconf-property-editor.h"
@@ -524,10 +525,11 @@ application_font_to_gconf (GConfPropertyEditor *peditor,
 					     GTK_DIALOG_MODAL,
 					     GTK_MESSAGE_WARNING,
 					     GTK_BUTTONS_NONE,
+					     "%s",
 					     warning_label);
 
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (warning_dialog),
-					      warning_label2);
+					      "%s", warning_label2);
 
     gtk_dialog_add_button (GTK_DIALOG (warning_dialog),
 			   _("Use previous font"), GTK_RESPONSE_CLOSE);
@@ -803,7 +805,6 @@ cb_details_response (GtkDialog *dialog, gint response_id)
 {
   if (response_id == GTK_RESPONSE_HELP) {
     capplet_help (GTK_WINDOW (dialog),
-		  "user-guide.xml",
 		  "goscustdesk-38");
   } else
     gtk_widget_hide (GTK_WIDGET (dialog));
